/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 Mudit Raj Gupta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Mudit Raj Gupta <mudit.raaj.gupta@gmail.com>
 */

#include "ns3/hla-helper.h"
#include "ns3/log.h"
#include "ns3/pointer.h"
#include <iostream>
#include <vector>

using namespace std;

namespace ns3 {

  NS_LOG_COMPONENT_DEFINE ("HlaHelper");

 
  int HlaHelper::m_ClassHandle;
  int HlaHelper::m_AttrHandle;
  vector<string> HlaHelper::m_Attributes;
  vector<string> HlaHelper::m_Values;
  vector<string> HlaHelper::m_id;
  vector<int> HlaHelper::m_NumAttr;
  vector <int> HlaHelper::m_Parent;
  int HlaHelper::m_Seconds;
  string HlaHelper::m_SyncPhrase;
  string HlaHelper::m_FomFile;
  string HlaHelper::m_FederationName;
  string HlaHelper::m_FederateName;
  double HlaHelper::m_RtiTimeResolution;
  int HlaHelper::m_StartValue;

HlaHelper::HlaHelper ()
{
  m_StartValue=0;
  m_Seconds=20;
  m_AttrHandle=0;
  m_ClassHandle=0;
  m_Seconds=0;
  m_SyncPhrase="ReadyToRun";
  m_FomFile="testfom.fed";
  m_FederationName="Example Federation";
  m_FederateName="ns3";
  m_RtiTimeResolution=1.0;
}

HlaHelper::~HlaHelper ()
{
}

void 
HlaHelper::SetSeconds(int seconds)
{
  m_Seconds = seconds; 
}

int 
HlaHelper::CreateSharedObject(string name, int numAttr)
{
  m_id.push_back(name);
  m_ClassHandle++;
  m_NumAttr.push_back(numAttr);
  return m_ClassHandle;
  //NS_LOG_UNCOND("Test");
}

int
HlaHelper::AddAttributeToSharedObject(string name, int cl)
{
  m_Attributes.push_back(name);
  m_Parent.push_back(cl);
  m_AttrHandle++;
  return m_AttrHandle;
  //NS_LOG_UNCOND("Test");
}

void 
HlaHelper::Start()
{
  NS_LOG_UNCOND("in Start");
  m_StartValue=1;
}

int 
HlaHelper::GetNumberOfAttributes()
{
  return m_AttrHandle;
}

int
HlaHelper::GetNumberofObjects()
{
  return m_ClassHandle;
}

string 
HlaHelper::GetAttributeValue(int attr)
{
  return m_Values[attr-1];
}

string 
HlaHelper::GetObjectofAttribute(int attr)
{
  return m_id[m_Parent[attr-1]];
}

string
HlaHelper::GetObjectName(int cl)
{
  return m_id[cl-1];
}

string
HlaHelper::GetAttributeName(int attr)
{
  return m_Attributes[attr-1];
}


void 
HlaHelper::SetSyncPhrase(string name)
{
  m_SyncPhrase = name; 
}

void 
HlaHelper::SetFomFile(string name)
{
  m_FomFile = name; 
}

void 
HlaHelper::SetFederationName(string name)
{
  m_FederationName = name; 
}

void 
HlaHelper::SetFederateName(string name)
{
  m_FederateName = name; 
}

void 
HlaHelper::SetRtiTimeResolution(double val)
{
  m_RtiTimeResolution = val; 
}

} // namespace ns3

