/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 Mudit Raj Gupta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Mudit Raj Gupta <mudit.raaj.gupta@gmail.com>
 */

#include <iostream>
#include <vector>
#include <string>

#ifndef HLA_HELPER_H
#define HLA_HELPER_H

using namespace std;

namespace ns3 {

/**
 * \ingroup hla
 * \brief class to help create a hla Helper
 */

class HlaHelper
{
public:
  /**
   * Construct a hla Helper to make it easier when working
   * with HLA.
   */
  HlaHelper ();

  /**
   * \internal
   * Destroy a hla Helper
   */
  ~HlaHelper ();

  void SetSeconds(int seconds);
  
  /**
   * Create an attribute of the created object
   *
   * \param name name of the attribute  
   * \param init initial value
   */
  int CreateSharedObject (string Name, int numattr);
  
  /**
   * Create an attribute of the created object
   *
   * \param name name of the attribute  
   * \param init initial value
   */
  int AddAttributeToSharedObject (string Name, int cl);
  

int GetNumberOfAttributes();
int GetNumberofObjects();
string GetAttributeValue(int attr);
string GetObjectofAttribute(int attr);
string GetObjectName(int cl);
string GetAttributeName(int attr);


  void Start();
  void SetSyncPhrase(string name); 
  void SetFomFile(string name);
  void SetFederationName(string name);
  void SetFederateName(string name);
  void SetRtiTimeResolution(double val);
  
  static int m_ClassHandle;
  static int m_AttrHandle;
  static vector<string> m_Attributes;
  static vector<string> m_Values;
  static vector<string> m_id;
  static vector<int> m_NumAttr;
  static vector <int> m_Parent;
  static int m_Seconds;
  static string m_SyncPhrase;
  static string m_FomFile;
  static string m_FederationName;
  static string m_FederateName;
  static double m_RtiTimeResolution;
  static int m_StartValue;
};

} // namespace ns3

#endif /* HLA_Helper_H */
