/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 Mudit Raj Gupta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Mudit Raj Gupta <mudit.raaj.gupta@gmail.com>
 */

#ifndef LINK_TO_RTI_H
#define LINK_TO_RTI_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <pthread.h>

#define PORT_NO 2201

namespace ns3 {

class LinkToRti
{

public:

/**
   * RTI_CLK, maintains the tick count from RTI
   * RTI_START flag is set when RTI grants a start of simulation
   * RTI_STOP floag is set when RTI issues a stop of simulation
   */

static int RTI_CLK;
static int RTI_START;
static int RTI_STOP;

// This method is invoked in a different thread, it acts as a link between the JAVA
//code and the C++ code
//void* rtiLink(void*);


/**
   * Time advance request is send to RTI through ns3Federate if no more events are scheduled 
   */
void reqTimeAdv(void);

/**
   * Method is called when time advance is granted to ns3
   */
void timeAdv(void);

/**
   * Starts simulation according to RTI 
   */
void startSim(void);

void InitializeFederate(void);

/**
   * Stops simulation according to RTI 
   */
void stopSim(void);

/**
   * Initialize Socket
   */
void init_sock(void);

/**
   * Close socket connection
   */
void close_sock(void);
   
  /**
   * Temporary Function
   */

void connectToRTI(void);

};

}// namespace ns3

#endif /*link-to-rti*/
