/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 Mudit Raj Gupta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This class handles all the callbacks for the ns3Federate. On each callback, it will
 * just create a string describing the event and add that string to a list of events that have
 * happened. At the end of processing, the contents of this list can be printed so we can see
 * exactly what events happened in what order.
 *
 * Author: Mudit Raj Gupta <mudit.raaj.gupta@gmail.com>
 *
 */

import java.util.ArrayList;
import java.util.List;

import hla.rti.EventRetractionHandle;
import hla.rti.LogicalTime;
import hla.rti.ReceivedInteraction;
import hla.rti.ReflectedAttributes;
import hla.rti.jlc.EncodingHelpers;
import hla.rti.jlc.NullFederateAmbassador;

import org.portico.impl.hla13.types.DoubleTime;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
	


public class ns3FedAmb extends NullFederateAmbassador
{

        //socket related variables
        public  Socket socketClient;
	public  OutputStream oStream=null;
	public  InputStream iStream=null;
	private String bufferReceiveString;
	private byte[] bufferReceiveByte=new byte[2000];
	public static String rtiAddressDefault="127.0.0.1";
	public String comnet="127.0.0.1";
	
        //counters and flags
	private int count=0;	
	public int msgNum=1;
	private int val=0;
	private int offset=0;
        //federate's state variables
	protected double federateTime        = 0.0;
	protected double federateLookahead   = 1.0;
	
	protected boolean isRegulating       = false;
	protected boolean isConstrained      = false;
	protected boolean isAdvancing        = false;
	
	protected boolean isAnnounced        = false;
	protected boolean isReadyToRun       = false;
	
	protected List<String> events        = new ArrayList<String>();

	//native methods
        //public native int intMethod(int n);
        public JavaLink jl =  new JavaLink(); 
        
        //constructor
	public ns3FedAmb() 
  	{
                System.setProperty("portico.lrc.connection", "org.portico.binding.jsop.lrc.JSOPClientConnection"); 
    		System.setProperty("portico.lrc.jsop.host", "127.0.0.1");
    		System.out.println(System.getProperty("portico.lrc.jsop.host"));
  	}	

	private double convertTime( LogicalTime logicalTime )
	{
		return ((DoubleTime)logicalTime).getTime();
	}
	
        //adding events to queue
	private synchronized void queue( String message )
	{
		events.add( message );
	}
	
	//CALL BACKS
	public void synchronizationPointRegistrationFailed( String label )
	{
		queue( "Failed to register sync point: " + label );
	}

	public void synchronizationPointRegistrationSucceeded( String label )
	{
		queue( "Successfully registered sync point: " + label );
	}

	public void announceSynchronizationPoint( String label, byte[] tag )
	{
		queue( "Synchronization point announced: " + label );
		if( label.equals(ns3Federate.READY_TO_RUN) )
			this.isAnnounced = true;
	}

	public void federationSynchronized( String label )
	{
		queue( "Federation Synchronized: " + label );
		if( label.equals(ns3Federate.READY_TO_RUN) )
			this.isReadyToRun = true;
	}

	// The RTI has informed us that time regulation is now enabled.
	public void timeRegulationEnabled( LogicalTime theFederateTime )
	{
		this.federateTime = convertTime( theFederateTime );
		this.isRegulating = true;
		queue( "Regulation Enabled: time=" + federateTime );
	}

	public void timeConstrainedEnabled( LogicalTime theFederateTime )
	{
		this.federateTime = convertTime( theFederateTime );
		this.isConstrained = true;
		queue( "Constrained Enabled: time=" + federateTime );
	}
        
        //Time Advance Granted
	public void timeAdvanceGrant( LogicalTime theTime )
	{
                //ns3FedAmb obj = new ns3FedAmb();
		Ns3TimeAdv();
                //jl.TimeAdvance();
                this.federateTime = convertTime( theTime );
		this.isAdvancing = false;
		queue( "ADVANCE("+federateTime+") Granted: time=" + federateTime );
	}

	public void discoverObjectInstance( int theObject,
	                                    int theObjectClass,
	                                    String objectName )
	{
		queue( "DISCOVERY objectHandle=" + theObject + ", classHandle=" +
		       theObjectClass + ", name=" + objectName );
	}

	public void reflectAttributeValues( int theObject,
	                                    ReflectedAttributes theAttributes,
	                                    byte[] tag )
	{
		// just pass it on to the other method for printing purposes
		// passing null as the time will let the other method know it
		// it from us, not from the RTI
		reflectAttributeValues( theObject, theAttributes, tag, null, null );

	}

	public void reflectAttributeValues( int theObject,
	                                    ReflectedAttributes theAttributes,
	                                    byte[] tag,
	                                    LogicalTime theTime,
	                                    EventRetractionHandle retractionHandle ) 
 	{
		String send,send1;
		String start = "REFLECTION(RO) objectHandle=";
		if( theTime != null )
			start = "REFLECTION("+theTime+") objectHandle=";
		StringBuilder builder = new StringBuilder( start );
		builder.append( theObject );
		builder.append( ", tag=" + EncodingHelpers.decodeString(tag) );
		builder.append( ", attributes=" + theAttributes.size() );
		if( theTime != null )
		{
			builder.append( ", time=" + convertTime(theTime) );
			builder.append( " {TIMESTAMPED}" );
		}

		try
		{
			send1 = "Time = "+federateTime+"::"+EncodingHelpers.decodeString(theAttributes.getValue(0))+"::";
			send=send1+EncodingHelpers.decodeString(theAttributes.getValue(1));
          		TellNs3(send);
		        queue(EncodingHelpers.decodeString(theAttributes.getValue(0)));
          		queue(EncodingHelpers.decodeString(theAttributes.getValue(1)));
		}
		catch(Exception e) 
      		{
         		System.out.println("ERRORE PS: " + e);
		}

                System.out.println( events.get(val) );
		val++;
		queue( builder.toString() );
	}

	public void removeObjectInstance( int theObject, byte[] userSuppliedTag )
	{	
		queue( "REMOVE objectHandle=" + theObject );
	}

	public void removeObjectInstance( int theObject,
	                                  byte[] userSuppliedTag,
	                                  LogicalTime theTime,
	                                  EventRetractionHandle retractionHandle )
	{
		queue( "REMOVE("+theTime+") objectHandle=" + theObject +
		       ", time=" + convertTime(theTime) + " {TIMESTAMPED}");
	}

        //sends the start sequence to ns3 and instructs it to start
	public void StartNs3()
	{
		count++;
		String oo="*****";
		try
		{
			oStream.write((oo).getBytes());}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}

        //everytime a time advance is received from the RTI ns3 is informed to make the RTI_CLK tick
        
        private void Ns3TimeAdv()
	{
		count++;
		String oo="#####";
		try
		{
			oStream.write((oo).getBytes());}
		catch (IOException e) 	
		{
			e.printStackTrace();
		}
	}
        
        //Message is send if a callback is received from RTI
	private void TellNs3(String s) 
  	{
    		System.out.println(count);
    		try 
      		{
			oStream.write((s).getBytes());
      		} 
    		catch (IOException e) 
      		{
			e.printStackTrace();
      		}
  	}

        //At the end of the simulation a stopping string is send to ns3
	public void StopNs3() 
	{
		try 
        	{
      		        String temp="+++++";
			this.oStream.write((temp).getBytes());
      		} 
    		catch (IOException e) 
      		{
			e.printStackTrace();
      		}
    		CloseSock();
  	}

        //The method handles requests from ns3
	public void AskNs3() 
  	{
    		try 
    		{
      			
      
     	 		while(!(new String(bufferReceiveByte).contains("***")))
			{			
     		     		this.iStream.read(bufferReceiveByte,offset,1);
		  		offset++;
			}
     		}	 
     		catch (IOException e) 
     		{
		         e.printStackTrace();
       		}	
     		System.out.println(bufferReceiveString);
    		bufferReceiveString=null;
    		bufferReceiveByte=new byte[2000];
  	}

        //At present a dummy method to mimic a time advance request
	public void readTimeAdv()
	{
                //String x;
	        //byte[] x=new byte[1];
try{		
iStream.read(bufferReceiveByte,offset,1);
offset++;
    }            
catch (IOException e) 
     		{
                        
		         e.printStackTrace();
       		}	
                //bufferReceiveString=new String(bufferReceiveByte);
		//if(bufferReceiveString.equals("1"))
    		//return true;
		//else 
                //return false;
  	}
	
        //Method to initialize sockets
	public void TellInitialize() 
  	{
    		try 
      		{
        		socketClient = new Socket(comnet, 2201);
        		oStream = socketClient.getOutputStream();
        		iStream = socketClient.getInputStream();
      		}
    		catch (Exception e) 
      		{
        		System.out.println("ERRORE PS: " + e);
      		}
	}

        //Method to close socket section
	public void CloseSock()
  	{
    		try 
      		{
        		socketClient.close();
      		}
    		catch (Exception e) 
      		{
         		System.out.println("ERRORE PS: " + e);
      		}
  	}

	
}
