/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2012 Mudit Raj Gupta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * It stores up information about all the messages it receives as callbacks
 * and logs them at the end of execution (in order).
 *
 * Authors: Mudit Raj Gupta <mudit.raaj.gupta@gmail.com>
 *
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.net.MalformedURLException;

import hla.rti.AttributeHandleSet;
import hla.rti.FederatesCurrentlyJoined;
import hla.rti.FederationExecutionAlreadyExists;
import hla.rti.FederationExecutionDoesNotExist;
import hla.rti.LogicalTime;
import hla.rti.LogicalTimeInterval;
import hla.rti.RTIambassador;
import hla.rti.RTIexception;
import hla.rti.ResignAction;
import hla.rti.SuppliedAttributes;
import hla.rti.SuppliedParameters;
import hla.rti.jlc.EncodingHelpers;
import hla.rti.jlc.RtiFactoryFactory;

import org.portico.impl.hla13.types.DoubleTime;
import org.portico.impl.hla13.types.DoubleTimeInterval;


public class ns3Federate
{

        private static JavaLink jl =  new JavaLink(); 
	//Length of simulation
	public static final int ITERATIONS = jl.GetSeconds();
       
	// The sync point all federates will sync up on before starting 
	public static final String READY_TO_RUN = jl.GetSyncPhrase();

        //For accessing other classes
	private RTIambassador rtiamb;
	private ns3FedAmb fedamb;
        
	private int numObj=0;
        //private int[] cl;
        //private int[] attr;
        //private int[] attributes;
        private int AttrCount=0;
        private int cl[] = new int[jl.GetObjectsCount()];
         private int attr[] = new int [jl.GetAttributesCount()];
         private AttributeHandleSet attributes[] = new AttributeHandleSet  [jl.GetObjectsCount()];
	//This is just a helper method to make sure all logging it output in the same form
 
private void log( String message )
	{
		System.out.println( jl.GetFederateName()+"  : " + message );
	}

	//This method will block until the user presses enter
	private void waitForUser()
	{
		log( " >>>>>>>>>> Press Enter to Continue <<<<<<<<<<" );
		BufferedReader reader = new BufferedReader( new InputStreamReader(System.in) );
		try
		{
			reader.readLine();
		}
		catch( Exception e )
		{
			log( "Error while waiting for user input: " + e.getMessage() );
			e.printStackTrace();
		}
	}
	
        // As all time-related code is Portico-specific, it is isolated into a 
	// single method. For a different RTI, change this code, rather than more 
        // code throughout the whole class.
	//

	private LogicalTime convertTime( double time )
	{
		// PORTICO SPECIFIC!!
		return new DoubleTime( time );
	}
	
	private LogicalTimeInterval convertInterval( double time )
	{
		// PORTICO SPECIFIC!!
		return new DoubleTimeInterval( time );
	}

	
	//This is the main simulation loop.
	public void runFederate( String federateName ) throws RTIexception
		{
		
                //create the RTIambassador 
		rtiamb = RtiFactoryFactory.getRtiFactory().createRtiAmbassador();

        // create the federation 
	// NOTE: some other federate may have already created the federation,
	// in that case, we'll just try and join it
        //

		try
		{
			File fom = new File( jl.GetFomFile() );
			rtiamb.createFederationExecution( jl.GetFederationName(),
			                                  fom.toURI().toURL() );
			log( "Created Federation" );
		}

		catch( FederationExecutionAlreadyExists exists )
		{
			log( "Didn't create federation, it already existed" );
		}

		catch( MalformedURLException urle )
		{
			log( "Exception processing fom: " + urle.getMessage() );
			urle.printStackTrace();
			return;
		}
		
		//join the federation 
		// create the federate ambassador and join the federation

		fedamb = new ns3FedAmb();
		rtiamb.joinFederationExecution( federateName, jl.GetFederationName(), fedamb );
		log( "Joined Federation as " + federateName );
                
                //announce the sync point 
		// announce a sync point to get everyone on the same page. if the point
		// has already been registered, we'll get a callback saying it failed,
		// but we don't care about that, as long as someone registered it
                rtiamb.registerFederationSynchronizationPoint( READY_TO_RUN, null );
//System.out.println(jl.stringMethod("Java"));
		// wait until the point is announced
		while( fedamb.isAnnounced == false )
		{
			rtiamb.tick();
		}

		// WAIT FOR USER TO KICK US OFF
		// So that there is time to add other federates, we will wait until the
		// user hits enter before proceeding. 
		waitForUser();

		//achieve the point and wait for synchronization
		// tell the RTI we are ready to move past the sync point and then wait
		// until the federation has synchronized on

		rtiamb.synchronizationPointAchieved( READY_TO_RUN );
		log( "Achieved sync point: " +READY_TO_RUN+ ", waiting for federation..." );
		while( fedamb.isReadyToRun == false )
		{
			rtiamb.tick();
		}

		//enable time policies 
		// in this section we enable/disable all time policies
		// note that this step is optional!
		enableTimePolicy();
		log( "Time Policy Enabled" );

		//publish and subscribe
		// in this section we tell the RTI of all the data we are going to
		// produce, and all the data we want to know about
                numObj=jl.GetObjectsCount();
                for(int i=1;i<=numObj;i++)
                {
                        addObject(jl.GetObjectName(i),jl.GetObjectAttributesCount(i),i);
                }   
      
 
		publishAndSubscribe();
		log( "Published and Subscribed" );

		//register an object to update 
		/*int objectHandle = registerObject();
		log( "Registered Object, handle=" + objectHandle );*/
		
                //Tell ns3 main code to intialize a socket
		fedamb.TellInitialize();

                //Tell ns3 to start the simulation
		fedamb.StartNs3();

		// do the main simulation loop 
		// here is where we do the meat of our work. in each iteration, we will
		// update the attribute values of the object we registered.

		for( int i = 0; i < ITERATIONS; i++ )
		{
			//request a time advance and wait until we get it
                       /* while (jl.RequestTimeAdvance()==0)
                        {}*/
                        fedamb.readTimeAdv();
			advanceTime(jl.GetRtiTimeResolution());/*fedamb.readTimeAdv() );*/
			log( "Time Advanced to " + fedamb.federateTime );
			if(i==(ITERATIONS-1))
				fedamb.StopNs3();
		}

		//delete the object we created 
		/*deleteObject( objectHandle );
		log( "Deleted Object, handle=" + objectHandle );*/

		//resign from the federation 
		rtiamb.resignFederationExecution( ResignAction.NO_ACTION );
		log( "Resigned from Federation" );

		// try and destroy the federation 
		// NOTE: we won't die if we can't do this because other federates
		//       remain. in that case we'll leave it for them to clean up
		
                try
		{
			rtiamb.destroyFederationExecution( "ExampleFederation" );
			log( "Destroyed Federation" );
		}
		catch( FederationExecutionDoesNotExist dne )
		{
			log( "No need to destroy federation, it doesn't exist" );
		}
		catch( FederatesCurrentlyJoined fcj )
		{
			log( "Didn't destroy federation, federates still joined" );
		}
		
		// print out all the events
		System.out.println( " ================= Ordered Event List =================" );
		for( int i = 0; i < fedamb.events.size(); i++ )
			System.out.println( "["+i+"]: " + fedamb.events.get(i) );
		System.out.println( " ======================================================" );
	}
	
	// Helper Methods 
	
        // This method will attempt to enable the various time related properties for
	// the federate
	//
	private void enableTimePolicy() throws RTIexception
	{
		// NOTE: Unfortunately, the LogicalTime/LogicalTimeInterval create code is
		//       Portico specific. 
		LogicalTime currentTime = convertTime( fedamb.federateTime );
		LogicalTimeInterval lookahead = convertInterval( fedamb.federateLookahead );
		
		// enable time regulation 
		this.rtiamb.enableTimeRegulation( currentTime, lookahead );

		// tick until we get the callback
		while( fedamb.isRegulating == false )
		{
			rtiamb.tick();
		}
		
		// enable time constrained 
		this.rtiamb.enableTimeConstrained();
		
		// tick until we get the callback
		while( fedamb.isConstrained == false )
		{
			rtiamb.tick();
		}
	}
	
	// This method will inform the RTI about the types of data that the federate will
	// be creating, and the types of data we are interested in hearing about as other
	// federates produce it.
	private void addObject(String obj, int at, int clval) throws RTIexception
        {
                System.out.println(("ObjectRoot."+obj));
                cl[clval-1]=rtiamb.getObjectClassHandle(("ObjectRoot."+obj)); 
                for(int i=1;i<=at;i++)
                {
                        System.out.println(jl.GetAttributeName(AttrCount+1));  
                        attr[AttrCount]=rtiamb.getAttributeHandle( jl.GetAttributeName(AttrCount+1) , cl[clval-1] );
                        AttrCount++;
                }
                attributes[clval-1] = RtiFactoryFactory.getRtiFactory().createAttributeHandleSet();
                for(int i=(AttrCount-at);i<=AttrCount;i++)
                        attributes[clval-1].add(attr[i]);
                rtiamb.publishObjectClass( cl[clval-1], attributes[clval-1] );
                rtiamb.subscribeObjectClassAttributes( cl[clval-1], attributes[clval-1] );
        }
        
      	private void publishAndSubscribe() throws RTIexception
	{
		// publish all attributes of ObjectRoot.PingPong 
		// before we can register instance of the object class ObjectRoot.PingPong and
		// update the values of the various attributes, we need to tell the RTI
	        // that we intend to publish this information
		// get all the handle information for the attributes of ObjectRoot.A

		/*int classHandle = rtiamb.getObjectClassHandle( "ObjectRoot.PingPong" );
		int MsgTypeHandle    = rtiamb.getAttributeHandle( "MsgType", classHandle );
		int MsgHandle    = rtiamb.getAttributeHandle( "Msg", classHandle );*/
		
		/*AttributeHandleSet attributes =
			RtiFactoryFactory.getRtiFactory().createAttributeHandleSet();
                for(int i=0;i<=AttrCount;i++)
        		attributes.add( attr[i] );*/
		//attributes.add( MsgHandle );
		
		// do the actual publication
/*		
rtiamb.publishObjectClass( classHandle, attributes );

		// subscribe to all attributes of ObjectRoot.PingPong
		// we also want to hear about the same sort of information as it is
		// created and altered in other federates, so we need to subscribe to it
		rtiamb.subscribeObjectClassAttributes( classHandle, attributes );
*/
	}
        
       // private void publishObject()
       // {
         
	
	// This method will register an instance of the class ObjectRoot.A and will
	// return the federation-wide unique handle for that instance. Later in the
	// simulation, we will update the attribute values for this instance
	
	private void registerObject() throws RTIexception
	{
		/*int classHandle = rtiamb.getObjectClassHandle( "ObjectRoot.PingPong" );
		return rtiamb.registerObjectInstance( classHandle );*/
	}
	
	// This method will request a time advance to the current time, plus the given
	// timestep. It will then wait until a notification of the time advance grant
	// has been received.
	 
	private void advanceTime( double timestep ) throws RTIexception
	{
		// request the advance
		fedamb.isAdvancing = true;
		LogicalTime newTime = convertTime( fedamb.federateTime + timestep );
		rtiamb.timeAdvanceRequest( newTime );
		
		// wait for the time advance to be granted. ticking will tell the
		// LRC to start delivering callbacks to the federate
		while( fedamb.isAdvancing )
		{
			rtiamb.tick();
		}
		
	}

	// This method will attempt to delete the object instance of the given
	// handle. We can only delete objects we created, or for which we own the
	// privilegeToDelete attribute.
	
	private void deleteObject( int handle ) throws RTIexception
	{
		//rtiamb.deleteObjectInstance( handle, null ); 
	}

	//Main Function
	public static void main( String[] args )
	{
		// setFederateName as ns3
		String federateName = jl.GetFederateName();
		if( args.length != 0 )
		{
			federateName = args[0];
		}
		
		try
		{
			// run the example federate
                        while(jl.GetStartSignal()==0)
                        {
jl.refresh();
System.out.println(jl.GetStartSignal());
                        }
                        
			new ns3Federate().runFederate( federateName );
		}
		catch( RTIexception rtie )
		{
			// an exception occurred, just log the information and exit
			rtie.printStackTrace();
		}
	}
}
